package com.bwie.auth.service.impl;

import cn.hutool.jwt.JWT;
import cn.hutool.jwt.JWTUtil;
import com.alibaba.fastjson.JSON;
import com.bwie.auth.feign.FeignSystem;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.request.UserReq;
import com.bwie.common.domain.response.UserRes;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.UUID;

/**
 * Description:
 *
 * @Author: sun-cool-boy
 * @Date: 2023/10/10
 * @info: 鉴权业务实现
 */
@Service
public class AuthServiceImpl implements AuthService {

    /**
     * 远程调用
     */
    @Resource
    private FeignSystem feignSystem;

    /**
     * 注入redis
     */
    @Resource
    private StringRedisTemplate stringRedisTemplate;

    /**
     * 注入http请求
     */
    @Resource
    private HttpServletRequest request;

    /**
     * 登录
     * @param userReq
     * @return
     */
    @Override
    public Result<UserRes> login(UserReq userReq) {
        if (StringUtils.isEmpty(userReq.getName())){
            return Result.error(null,"用户名不能为空");
        }
        Result<UserRes> userResResult = feignSystem.selectUser(userReq);
        UserRes data = userResResult.getData();
        if (!data.getPwd().equals(userReq.getPwd())){
            return Result.error(null,"密码不正确");
        }
        //生成token令牌
        HashMap<String, Object> map = new HashMap<>();
        String userKey = UUID.randomUUID().toString().replaceAll("-", "");
        map.put(JwtConstants.USER_KEY,userKey);
        String token = JwtUtils.createToken(map);
        data.setToken(token);
        //用户信息存入redis
        stringRedisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY + userKey, JSON.toJSONString(data));

        return Result.success(data);
    }
    /**
     * 获取用户信息
     * @return
     */
    @Override
    public Result<UserRes> info() {
        String userKey = JwtUtils.getUserKey(request.getHeader("token"));
        UserRes userRes = JSON.parseObject(stringRedisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey), UserRes.class);
        return Result.success(userRes);
    }

    @Override
    public Result logout() {
        String userKey = JwtUtils.getUserKey(request.getHeader("token"));
        stringRedisTemplate.delete(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return Result.success();
    }
}
