package com.bwie.gateway.filters;

import com.bwie.common.constants.TokenConstants;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import java.util.List;

/**
 * Description:
 *
 * @Author: sun-cool-boy
 * @Date: 2023/10/6
 * @info:
 */
@Component
public class GatewayFilter implements GlobalFilter, Ordered {

    /**
     * redis
     */
    @Resource
    private StringRedisTemplate stringRedisTemplate;

    /**
     * 白名单
     */
    @Resource
    private IgnoreWhiteConfig ignoreWhiteConfig;

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        String path = exchange.getRequest().getURI().getPath();
        //校验白名单
        if (StringUtils.matches(path,ignoreWhiteConfig.getWhites())){
            //放行
            return chain.filter(exchange);
        }
        //token校验
        String token = exchange.getRequest().getHeaders().getFirst("token");
        if (StringUtils.isEmpty(token)){
            return GatewayUtils.errorResponse(exchange,"token不能为空");
        }
        try {
            JwtUtils.parseToken(token);
        }catch (Exception e){
            return GatewayUtils.errorResponse(exchange,"token不合法");
        }
        //判断redis中是否存在
        if (!stringRedisTemplate.hasKey(TokenConstants.LOGIN_TOKEN_KEY + JwtUtils.getUserKey(token))){
            return GatewayUtils.errorResponse(exchange,"身份过期 重新登陆");
        }
        //放行
        return chain.filter(exchange);
    }

    @Override
    public int getOrder() {
        return 0;
    }
}
