package com.bwie.server.consumer;

import com.bwie.common.domain.BorrowMoney;
import com.bwie.common.domain.request.BorrowMoneyReq;
import com.bwie.server.feign.FeignES;
import com.bwie.server.mapper.BorrowMoneyMapper;
import com.rabbitmq.client.Channel;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;

/**
 * Description:
 *
 * @Author: sun-cool-boy
 * @Date: 2023/10/10
 * @info:
 */
@Component
public class RabbitConsumer {

    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private FeignES feignES;

    @Resource
    private BorrowMoneyMapper borrowMoneyMapper;

    @RabbitListener(queuesToDeclare = @Queue(value = "one"))
    public void listen(String msg, Message message, Channel channel){

        String messageId = message.getMessageProperties().getMessageId();
        Long one = stringRedisTemplate.opsForSet().add("one", messageId);
        long deliveryTag = message.getMessageProperties().getDeliveryTag();
        if (one>0){
            //根据msg 也就是id查询当前借款表数据状态 被借出不操作 否则状态改为未放款
            BorrowMoneyReq borrowMoneyReq = new BorrowMoneyReq();
            borrowMoneyReq.setId(Integer.parseInt(msg));
            List<BorrowMoney> list = borrowMoneyMapper.selectBorrowMoney(borrowMoneyReq);
            BorrowMoney borrowMoney = list.get(0);
            //判断状态
            if (borrowMoney.getStatus()==0){
                //代表没人资助 更改状态并发消息
                borrowMoneyMapper.upd(borrowMoney.getId(),2);
                //es更新数据
                feignES.addBatch();
                //没手机就控制台打印一下了
                System.out.println("您的借款暂时无人资助 ");
            }
            //确认收到消息以及不批量确认
            try {
                channel.basicAck(deliveryTag,false);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }else {
            //消息重复
            try {
                channel.basicReject(deliveryTag,true);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
