package com.bwie.server.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.domain.InterestRate;
import com.bwie.common.result.Result;
import com.bwie.server.config.RedisMeta;
import com.bwie.server.mapper.InterestRateMapper;
import com.bwie.server.service.InterestRateService;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Description:
 *
 * @Author: sun-cool-boy
 * @Date: 2023/10/10
 * @info: 利率业务层
 */
@Service
public class InterestRateServiceImpl implements InterestRateService {

    /**
     * 注入redis 通过redis获取利率信息
     */
    @Resource
    private StringRedisTemplate stringRedisTemplate;


    /**
     * 通过redis获取利率信息
     * @return
     */
    @Override
    public Result<List<InterestRate>> selectInterestRate() {
        //获取集合信息
        List<String> range = stringRedisTemplate.opsForList().range(RedisMeta.INTEREST_RATE_INFO, 0, -1);
        //断言集合不能为空
        assert range != null;
        //转类型并返回
        List<InterestRate> collect = range
                .stream()
                .map(obj -> JSON.parseObject(obj, InterestRate.class))
                .collect(Collectors.toList());
        return Result.success(collect);
    }
}
